from installclass import BaseInstallClass
from constants import *
from filer import *
from flags import flags
import os
import isys
import iutil
import packages
import shutil
import types
import yuminstall
try:
    import instnum
except ImportError:
    instnum = None

import gettext
_ = lambda x: gettext.ldgettext("anaconda", x)

import logging
log = logging.getLogger("anaconda")

# FIXME: geninitrd is dumb as hell
class ModuleHelper(object):
    types = {}
    def loadType(self, idetype):
        if not self.types.has_key(idetype):
            mods = {}
            f = open('/usr/share/pci-database/%s.pci' % idetype)
            for line in f:
                id, mod = line.split()[:2]
                mods[id] = mod
            f.close()
            self.types[idetype] = mods
        return self.types[idetype]
    def getModulesForType(self, idetype):
        mods = []
        data = self.loadType(idetype)
        f = open('/proc/bus/pci/devices')
        for line in f:
            id = line.split()[1]
            if data.has_key(id):
                mods.append(data[id])
        f.close()
        return mods
    def getModprobeConf(self):
        # geninitrd treats sata as scsi
        for m in self.getModulesForType('sata') + self.getModulesForType('scsi'):
            yield 'alias scsi_hostadapter %s\n' % m
        for m in self.getModulesForType('ide'):
            yield 'alias ide_hostadapter %s\n' % m
    def writeModprobeConf(self, instPath):
        f = open(instPath + "/etc/modprobe.conf", "a")
        for l in self.getModprobeConf():
            f.write(l)
        f.close()

# custom installs are easy :-)
class InstallClass(BaseInstallClass):
    # name has underscore used for mnemonics, strip if you dont need it
    id = "pld"
    name = N_("PLD Linux")
    _description = N_("The default installation of %s includes a set of "
                     "software applicable for general internet usage. "
                     "What additional tasks would you like your system "
                     "to include support for?")
    _descriptionFields = (productName,)
    sortPriority = 10001
    allowExtraRepos = True

    repopaths = {
        "PLD Th - architecture specific": [
            "PLD/i686/RPMS",
        ],
        "PLD Th - generic packages": [
            "PLD/noarch/RPMS"
        ]
    }
    tasks = [
        (N_("Base"), [
            "basic",
            "base",
        ]),
        (N_("GNOME Desktop"), [
            "gnome",
            "gnome_complete",
            "gnome_games",
            "gnome_themes",
        ]),
        (N_("KDE Desktop"), [
            "kde_kdepim",
            "kde_kdeedu",
            "kde_multimedia",
            "kde_koffice",
            "kde_network",
            "kde_graphics",
            "kde_admin",
            "kde_games",
            "kde_look"
        ]),
        (N_("Basic IceWM"), [
            "icewm",
        ]),
        (N_("WindowMaker"), [
            "wmaker",
        ]),
        (N_("General Development Tools"), [
            "devel"
        ]),
        (N_("Java Development Tools"), [
            "java"
        ]),
    ]

    def setInstallData(self, anaconda):
        BaseInstallClass.setInstallData(self, anaconda)
        if not anaconda.isKickstart:
            BaseInstallClass.setDefaultPartitioning(self, 
                                                    anaconda.id.partitions,
                                                    CLEARPART_TYPE_LINUX)

    def setSteps(self, anaconda):
        dispatch = anaconda.dispatch
        BaseInstallClass.setSteps(self, anaconda);
        dispatch.skipStep("partition")
        dispatch.skipStep("regkey")

    def getPackagePaths(self, uri):
        rc = {}
        for (name, path) in self.repopaths.items():
            if not type(uri) == types.ListType:
                uri = [uri,]
            if not type(path) == types.ListType:
                path = [path,]

            lst = []
            for i in uri:
                for p in path:
                    lst.append("%s/%s" % (i, p))

            rc[name] = lst

        log.info("package paths is %s" %(rc,))
        return rc

    def postAction(self, anaconda):
        # FIXME: we only need to write fstab here because of rpm bug
        # see https://bugs.launchpad.net/pld-linux/+bug/262985
        upgrade = anaconda.id.getUpgrade()
        if not upgrade:
            anaconda.id.fsset.write(anaconda.rootPath)
            # fresh installs require modprobe.conf
            ModuleHelper().writeModprobeConf(anaconda.rootPath)
        log.info("Going to bind /dev, /proc and /sys to run geninitrd")
        isys.mount("/dev", "%s/dev" %(anaconda.rootPath,), bindMount = 1)
        isys.mount("/proc", "%s/proc" %(anaconda.rootPath,), bindMount = 1)
        isys.mount("/sys", "%s/sys" %(anaconda.rootPath,), bindMount = 1)
        # find all installed kernels and try to generate initrd for each one
        kernels = packages.rpmKernelVersionList(anaconda.rootPath)
        for kernel in kernels:
            kernelVer = kernel[0]
            log.info("Running geninitrd for kernel %s" % kernelVer)
            # FIXME: force romfs due to geninitrd bug
            # see https://bugs.launchpad.net/pld-linux/+bug/263149
            iutil.execWithRedirect("/sbin/geninitrd", ["-v", "--initrdfs=romfs", "-f", "/boot/initrd-%s.gz" % kernelVer, kernelVer], root = anaconda.rootPath)
        log.info("Umounting /dev, /proc and /sys")
        isys.umount("%s/dev" %(anaconda.rootPath,))
        isys.umount("%s/proc" %(anaconda.rootPath,))
        isys.umount("%s/sys" %(anaconda.rootPath,))
        # end geninitrd workaround
        # try to get xorg into working state
        if os.path.exists("/etc/X11/xorg.conf") \
        and not os.path.exists("%s/etc/X11/xorg.conf" % anaconda.rootPath) \
        and os.path.exists("%s/etc/X11" % anaconda.rootPath):
            log.info("Copying xorg.conf to the new system")
            shutil.copyfile("/etc/X11/xorg.conf", "%s/etc/X11/xorg.conf" % anaconda.rootPath)

    def handleRegKey(self, key, intf, interactive = True):
        pass

    def getBackend(self):
        return yuminstall.YumBackend

    def __init__(self):
        BaseInstallClass.__init__(self)

